#File type: extension <class page> set
#By Junxiang H., 2023/07/9
#wacmk.com/cn Tech. Supp.

#This script updates automaticly! Do not Modify!
#Update time:2023-10-22 14:55:09

Pages={}
import ShockFinder.Addon.GUI.XUIlib.pages.Help as Help
Pages["Help"]=Help
import ShockFinder.Addon.GUI.XUIlib.pages.Figure as Figure
Pages["Figure"]=Figure
import ShockFinder.Addon.GUI.XUIlib.pages.Analyze as Analyze
Pages["Analyze"]=Analyze
import XenonUI.XUIlib.pages.Exit as Exit
Pages["Exit"]=Exit
